#!/usr/bin/env python
"""Tests for ltdmason.s3upload.

   These tests required the following environment variables:

   ``LTD_MASON_TEST_AWS_ID``
      AWS access key ID

   ``LTD_MASON_TEST_AWS_SECRET``
      AWS secret access key

   ``LTD_MASON_TEST_BUCKET``
      Name of an S3 bucket that already exists and can be used for testing.

   The tests will be skipped if they are not available.

   Note that this test will create a random uuid4) directory at the root of
   ``LTD_MASON_TEST_BUCKET``, though the test harness will attempt to delete
   it.
"""

from __future__ import (division, absolute_import, print_function,
                        unicode_literals)
from builtins import *  # NOQA
from future.standard_library import install_aliases
install_aliases()  # NOQA

import os
import shutil
import tempfile
import uuid
import logging
import mimetypes

import pytest
import boto3
import requests
from ltdmason import s3upload

log = logging.getLogger(__name__)


@pytest.mark.skipif(os.getenv('LTD_MASON_TEST_AWS_ID') is None or
                    os.getenv('LTD_MASON_TEST_AWS_SECRET') is None or
                    os.getenv('LTD_MASON_TEST_BUCKET') is None,
                    reason='Set LTD_MASON_TEST_AWS_ID, '
                           'LTD_MASON_TEST_AWS_SECRET and '
                           'LTD_MASON_TEST_BUCKET')
def test_s3upload(request):
    """Integration test of s3upload and synchronnization upon file deletes.

    This is a test that operates more as an integrations test than a unit
    test.
    """
    bucket_name = os.getenv('LTD_MASON_TEST_BUCKET')
    aws_credentials = {
        'aws_access_key_id': os.getenv('LTD_MASON_TEST_AWS_ID'),
        'aws_secret_access_key': os.getenv('LTD_MASON_TEST_AWS_SECRET')}
    session = boto3.session.Session(**aws_credentials)

    logging.basicConfig(level=logging.ERROR)

    logging.getLogger('ltdmason.s3upload').level = logging.INFO
    logging.getLogger(__name__).level = logging.INFO

    paths = [
        'file1.txt',
        'file2.txt',
        'dir1/file11.txt',
        'dir1/file12.txt',
        'dir1/dir11/file111.txt',
        'dir1/dir11/file112.txt',
        'dir2/file21.txt',
        'dir2/file22.txt',
    ]

    temp_dir = tempfile.mkdtemp()
    temp_bucket_dir = str(uuid.uuid4())

    def cleanup():
        print("Cleaning up the bucket")
        _clean_bucket(session, bucket_name, temp_bucket_dir)
    request.addfinalizer(cleanup)

    _create_test_files(temp_dir, paths)

    surrogate_key = 'test-surrogate-key'
    cache_control_max_age = 3600
    s3upload.upload(bucket_name,
                    temp_bucket_dir,
                    temp_dir,
                    surrogate_key=surrogate_key,
                    upload_dir_redirect_objects=True,
                    cache_control_max_age=cache_control_max_age,
                    **aws_credentials)

    _test_objects_exist(session, bucket_name, temp_bucket_dir, paths)
    _test_content_types(session, bucket_name, temp_bucket_dir)
    _test_directory_redirects(session, bucket_name, temp_bucket_dir, paths)

    expected_headers = {
        'x-amz-meta-surrogate-key': surrogate_key,
        'Cache-Control': 'max-age={0:d}'.format(cache_control_max_age)}
    _test_headers(session, bucket_name, temp_bucket_dir, expected_headers)

    # Remove some files (on filesystem and in path manifest)
    shutil.rmtree(os.path.join(temp_dir, 'dir1'))
    paths.remove('dir1/file11.txt')
    paths.remove('dir1/file12.txt')
    paths.remove('dir1/dir11/file111.txt')
    paths.remove('dir1/dir11/file112.txt')
    os.remove(os.path.join(temp_dir, 'file2.txt'))
    paths.remove('file2.txt')

    s3upload.upload(bucket_name,
                    temp_bucket_dir,
                    temp_dir,
                    **aws_credentials)
    _test_objects_exist(session, bucket_name, temp_bucket_dir, paths)

    shutil.rmtree(temp_dir)


def _create_test_files(temp_dir, file_list):
    for path in file_list:
        _write_file(temp_dir, path)


def _write_file(root_dir, rel_path):
    filepath = os.path.join(root_dir, rel_path)
    try:
        os.makedirs(os.path.dirname(filepath))
    except OSError:
        # probably exists; pickup real error on write
        pass

    with open(filepath, 'w') as f:
        f.write('Content of {0}'.format(os.path.basename(filepath)))


def _test_objects_exist(session, bucket_name, bucket_root, file_list):
    s3 = session.resource('s3')
    bucket = s3.Bucket(bucket_name)

    # dirnames = _file_list_dirnames(file_list, bucket_root)

    bucket_objects = []
    for obj in bucket.objects.filter(Prefix=bucket_root):
        # skip directory objects
        if not os.path.splitext(obj.key)[-1]:
            continue
        bucket_objects.append(obj.key)

    assert len(bucket_objects) == len(file_list)
    for path in file_list:
        found = False
        for bucket_path in bucket_objects:
            if bucket_path.endswith(path):
                found = True
                continue
        if not found:
            log.error('{0} not found in bucket'.format(bucket_path))
            assert False


def _test_headers(session, bucket_name, bucket_root, expected_headers):
    """Generically test that header key-value pairs in `expected_headers`
    actually are served by S3.
    """
    s3 = session.resource('s3')
    bucket = s3.Bucket(bucket_name)

    # see http://stackoverflow.com/a/34698521 for making object URLs
    bucket_location = s3.meta.client.get_bucket_location(Bucket=bucket_name)

    for obj in bucket.objects.filter(Prefix=bucket_root):
        object_url = "https://s3-{0}.amazonaws.com/{1}/{2}".format(
            bucket_location['LocationConstraint'],
            bucket_name,
            obj.key)
        r = requests.head(object_url)
        # skip directory redirect objects
        if 'x-amz-ltd-redirect' in r.headers:
            continue
        for key, expected_value in expected_headers.items():
            assert r.headers[key] == expected_value


def _test_content_types(session, bucket_name, bucket_root):
    """Verify that the expected Content-Type header was set."""
    s3 = session.resource('s3')
    bucket = s3.Bucket(bucket_name)

    # AWS api doesn't give Content-Type header, so we'll test it directly
    # via HTTP
    # see http://stackoverflow.com/a/34698521 for making object URLs
    bucket_location = s3.meta.client.get_bucket_location(Bucket=bucket_name)

    for obj in bucket.objects.filter(Prefix=bucket_root):
        # skip directory redirect objects
        if 'ltd-redirect' in obj.Object().metadata:
            continue
        guess, _ = mimetypes.guess_type(obj.key)
        object_url = "https://s3-{0}.amazonaws.com/{1}/{2}".format(
            bucket_location['LocationConstraint'],
            bucket_name,
            obj.key)
        if guess is not None:
            r = requests.head(object_url)
            assert r.headers['content-type'] == guess


def _file_list_dirnames(file_list, bucket_root):
    dirnames = [bucket_root]
    for path in file_list:
        _d = os.path.dirname(path)
        if len(_d) > 0:
            dirnames.append(os.path.join(bucket_root, _d))
    dirnames = list(set(dirnames))
    return dirnames


def _test_directory_redirects(session, bucket_name, bucket_root, file_list):
    """Verify that the directory redirect objects exist."""
    # Make a list of all directories, including the root directory
    dirnames = _file_list_dirnames(file_list, bucket_root)

    # see http://stackoverflow.com/a/34698521 for making object URLs
    s3 = session.resource('s3')
    # bucket = s3.Bucket(bucket_name)
    bucket_location = s3.meta.client.get_bucket_location(Bucket=bucket_name)

    for dirname in dirnames:
        # Try to request the object
        obj = s3.Object(bucket_name, dirname)
        object_url = "https://s3-{0}.amazonaws.com/{1}/{2}".format(
            bucket_location['LocationConstraint'],
            bucket_name,
            obj.key)
        r = requests.head(object_url)
        assert r.headers['x-amz-meta-dir-redirect'] == 'true'


def _clean_bucket(session, bucket_name, root_path):
    s3 = session.resource('s3')
    bucket = s3.Bucket(bucket_name)

    # Normalize directory path for searching patch prefixes of objects
    if not root_path.endswith('/'):
        root_path += '/'

    key_objects = [{'Key': obj.key}
                   for obj in bucket.objects.filter(Prefix=root_path)]
    if len(key_objects) == 0:
        return
    delete_keys = {'Objects': []}
    delete_keys['Objects'] = key_objects
    # based on http://stackoverflow.com/a/34888103
    s3.meta.client.delete_objects(Bucket=bucket.name,
                                  Delete=delete_keys)



"""Test the Product build process using a mock documentation.

See mock_manifest.yaml; the doc repo is github.com/lsst-sqre/mock-doc and
the packages are embedded in this repo's test_data/ directory.
"""

from __future__ import (division, absolute_import, print_function,
                        unicode_literals)
from builtins import *  # NOQA
from future.standard_library import install_aliases
install_aliases()  # NOQA

import os
import tempfile
import shutil

import pkg_resources
import pytest
import sh
import ruamel.yaml

from ltdmason.product import Product
from ltdmason.manifest import Manifest


@pytest.fixture(scope='session')
def mock_manifest():
    """Provide the mock_manifest.yaml test file as a string."""
    resource_args = (__name__, 'mock_manifest.yaml')
    assert pkg_resources.resource_exists(*resource_args)
    yaml_str = pkg_resources.resource_string(*resource_args)

    # Patch the mock manifest to turn embedded package dir paths into
    # absolute paths
    yaml_data = ruamel.yaml.load(yaml_str)
    pkg_names = ('alpha', 'beta')
    test_dir = os.path.dirname(__file__)
    for n in pkg_names:
        p = os.path.join(test_dir, yaml_data['packages'][n]['dir'])
        yaml_data['packages'][n]['dir'] = p
    yaml_str = ruamel.yaml.dump(yaml_data)

    return yaml_str


@pytest.fixture(scope='session')
def product(request, mock_manifest):
    """Run the full product build pipeline.

    Since this fixture is session scoped it will only be run once; the tests
    then inspect the built product.

    The built documentation is automatically cleaned up with a finalizer.
    """
    build_dir = tempfile.mkdtemp()
    manifest = Manifest(mock_manifest)
    product = Product(manifest, build_dir)
    product.clone_doc_repo()
    product.link_package_repos()
    product.build_sphinx()

    def cleanup():
        shutil.rmtree(build_dir)
    request.addfinalizer(cleanup)

    return product


def test_clone_doc_repo(product):
    """Test how the doc repo was cloned."""
    manifest = product.manifest

    assert os.path.exists(product.doc_dir)
    # verify the correct ref is checked out
    git = sh.git.bake(_cwd=product.doc_dir)
    ref = str(git('symbolic-ref', 'HEAD').strip())
    print("doc repo ref:", ref, manifest.doc_repo_ref)
    assert ref.endswith(manifest.doc_repo_ref)


def test_link_package_repos(product):
    """Test links made from doc repo to package repos."""
    print(sh.ls('-al', product.doc_dir))

    for package_name, package_data in product.manifest.packages.items():
        print(package_name)
        # test that the package's directory exists in docs
        package_dir = os.path.join(product.build_dir,
                                   product.manifest.doc_repo_name,
                                   str(package_name))
        print(package_dir)
        assert os.path.isdir(package_dir)

        # test that the packages doc/_static directory is linked
        package_static_dir = os.path.join(package_data['dir'],
                                          'doc', '_static', package_name)
        if os.path.exists(package_static_dir):
            doc_package_static_dir = os.path.join(product.doc_dir,
                                                  '_static', str(package_name))
            assert os.path.islink(doc_package_static_dir)
            assert os.path.isdir(doc_package_static_dir)

        # test that individual entities of a package's doc (aside from _static)
        # are linked
        source_dir = os.path.join(package_data['dir'], 'doc')
        print('source_dir', source_dir)
        print(os.listdir(source_dir))
        target_dir = os.path.join(product.doc_dir, str(package_name))
        for entity in os.listdir(source_dir):
            print('entity', entity)
            if entity in product.package_excludes:
                continue
            link_name = os.path.join(target_dir, entity)
            assert os.path.islink(link_name)
            assert os.path.lexists(link_name)


def test_sphinx_build(product):
    assert os.path.exists(os.path.join(product.doc_dir, '_build', 'html'))



"""Test ltdmason.manifest."""

from __future__ import (division, absolute_import, print_function,
                        unicode_literals)
from builtins import *  # NOQA
from future.standard_library import install_aliases
install_aliases()  # NOQA

import pkg_resources
import pytest
from jsonschema import ValidationError
import ruamel.yaml

from ltdmason.manifest import Manifest, TravisManifest


@pytest.fixture
def demo_manifest():
    resource_args = (__name__, 'demo_manifest.yaml')
    assert pkg_resources.resource_exists(*resource_args)
    yaml_data = pkg_resources.resource_string(*resource_args)
    return yaml_data


@pytest.mark.xfail
def test_manifest_roundtrip(demo_manifest):
    # Can't get perfect round-trip equality of the YAML
    manifest = Manifest(demo_manifest)
    assert manifest.yaml == demo_manifest


def test_doc_repo_data(demo_manifest):
    manifest = Manifest(demo_manifest)
    assert 'pipelines_docs' == manifest.doc_repo_name
    assert 'master' == manifest.doc_repo_ref
    assert 'https://github.com/lsst-sqre/pipelines_docs.git' \
        == manifest.doc_repo_url
    assert 'lsst_apps' == manifest.product_name
    assert 'b1' == manifest.build_id
    assert 'jonathansick' == manifest.requester_github_handle
    assert ['master'] == manifest.refs
    assert 'https://github.com/lsst/afw.git' == manifest.packages['afw']['url']
    assert '/mnt/stack_docs/lsstsw/stack/Linux64/afw/2015_10.0-14-g7c5ed66' \
        == manifest.packages['afw']['dir']
    assert 'master' == manifest.packages['afw']['ref']


@pytest.mark.parametrize('key', ['product_name', 'build_id', 'refs',
                                 'requester_github_handle', 'doc_repo',
                                 'packages'])
def test_missing_manifest_fields(demo_manifest, key):
    """Test that exceptions are raised when a manifest is missing required
    fields.
    """
    manifest = ruamel.yaml.load(demo_manifest)
    del manifest[key]
    with pytest.raises(ValidationError):
        Manifest.validate(manifest)


@pytest.mark.parametrize('key', ['url', 'ref'])
def test_missing_manifest_doc_repo_fields(demo_manifest, key):
    """Test that exceptions are raised when a manifest is missing required
    fields in the 'doc_repo' object
    """
    manifest = ruamel.yaml.load(demo_manifest)
    del manifest['doc_repo'][key]
    with pytest.raises(ValidationError):
        Manifest.validate(manifest)


@pytest.mark.parametrize('key', ['dir', 'url', 'ref'])
def test_missing_manifest_package_fields(demo_manifest, key):
    """Test that exceptions are raised when a manifest is missing required
    fields in the 'packages.packages.afw' object.
    """
    manifest = ruamel.yaml.load(demo_manifest)
    del manifest['packages']['afw'][key]
    with pytest.raises(ValidationError):
        Manifest.validate(manifest)


def test_bad_refs_type(demo_manifest):
    """Test that refs is a list."""
    manifest = ruamel.yaml.load(demo_manifest)
    manifest['refs'] = 'master'
    with pytest.raises(ValidationError):
        Manifest.validate(manifest)


def test_travis_manifest(monkeypatch):
    monkeypatch.setenv('TRAVIS_BRANCH', 'master')
    monkeypatch.setenv('TRAVIS_REPO_SLUG', 'lsst-sqre/ltd-keeper')
    monkeypatch.setenv('LTD_MASON_PRODUCT', 'ltd-keeper')

    manifest = TravisManifest()
    assert manifest.doc_repo_url \
        == 'https://github.com/lsst-sqre/ltd-keeper.git'
    assert manifest.doc_repo_ref == 'master'
    assert manifest.product_name == 'ltd-keeper'
    assert manifest.build_id is None
    assert manifest.requester_github_handle is None
    assert manifest.refs == ['master']
    assert len(manifest.packages) == 0


def test_travis_manifest_missing_envvars(monkeypatch):
    monkeypatch.delenv('TRAVIS_BRANCH', raising=False)
    monkeypatch.delenv('TRAVIS_REPO_SLUG', raising=False)
    monkeypatch.delenv('LTD_MASON_PRODUCT', raising=False)

    manifest = TravisManifest()

    with pytest.raises(RuntimeError):
        manifest.doc_repo_url
    with pytest.raises(RuntimeError):
        manifest.doc_repo_ref
    with pytest.raises(RuntimeError):
        manifest.product_name
    with pytest.raises(RuntimeError):
        manifest.refs



"""Tests for the ltdmason/uploader module."""

import os
from base64 import b64encode
try:
    from unittest import mock
except ImportError:
    import mock
import responses
import pkg_resources
import pytest

from ltdmason.manifest import Manifest
from ltdmason.uploader import (_register_build, _confirm_upload, KeeperError,
                               upload_via_keeper, get_keeper_token,
                               read_keeper_credentials,
                               read_aws_credentials)


@responses.activate
def test_get_keeper_token():
    """Test getting a token from LTD Keeper."""
    expected_json = {'token': 'shake-it-off-shake-it-off'}
    responses.add(
        responses.GET,
        'http://localhost:5000/token',
        json=expected_json,
        status=200)

    _auth_header = 'Basic ' + b64encode(('user:pass')
                                        .encode('utf-8')).decode('utf-8')

    token = get_keeper_token('http://localhost:5000', 'user', 'pass')

    assert responses.calls[0].request.url == 'http://localhost:5000/token'
    assert responses.calls[0].request.headers['Authorization'] \
        == _auth_header
    assert token == 'shake-it-off-shake-it-off'


@responses.activate
def test_get_keeper_token_error():
    """Test with server error."""
    expected_json = {'token': 'shake-it-off-shake-it-off'}
    responses.add(
        responses.GET,
        'http://localhost:5000/token',
        json=expected_json,
        status=401)

    with pytest.raises(RuntimeError):
        get_keeper_token('http://localhost:5000', 'user', 'pass')


def test_read_keeper_credentials(monkeypatch):
    """Basic test of read_keeper_credentials()"""
    _url = 'http://ltd-keeper.lsst.codes'
    _user = 'user'
    _pass = 'pass'
    monkeypatch.setenv('LTD_KEEPER_URL', _url)
    monkeypatch.setenv('LTD_KEEPER_USER', _user)
    monkeypatch.setenv('LTD_KEEPER_PASSWORD', _pass)

    c = read_keeper_credentials()
    assert c['keeper_url'] == _url
    assert c['keeper_username'] == _user
    assert c['keeper_password'] == _pass


@pytest.mark.parametrize('missing', ['LTD_KEEPER_URL', 'LTD_KEEPER_USER',
                                     'LTD_KEEPER_PASSWORD'])
def test_read_keeper_credentials_missing(monkeypatch, missing):
    """Test that read_keeper_credentials raises a RuntimeError when a necessary
    environment variable is missing.
    """
    _url = 'http://ltd-keeper.lsst.codes'
    _user = 'user'
    _pass = 'pass'
    monkeypatch.setenv('LTD_KEEPER_URL', _url)
    monkeypatch.setenv('LTD_KEEPER_USER', _user)
    monkeypatch.setenv('LTD_KEEPER_PASSWORD', _pass)

    monkeypatch.delenv(missing)

    with pytest.raises(RuntimeError):
        read_keeper_credentials()


def test_read_aws_credentials_envvar(monkeypatch):
    """Test read_aws_credentials() if through environment variables.

    read_aws_credentials() is challenging to test because we not only test the
    presence of an environment variable, but also the absence. The delvar
    monkeypatch method only removes the monkeypatched var; an actual
    environment variable of the same name can still leak through. Instead we
    put the logic of omitting variables into a function that replaces
    os.getenv.
    """
    v = {'LTD_MASON_AWS_ID': 'key-id',
         'LTD_MASON_AWS_SECRET': 'key-secret',
         'LTD_MASON_AWS_PROFILE': 'aws-profile'}
    missing = ('LTD_MASON_AWS_PROFILE',)

    def patch_envar(x):
        if x in missing:
            return None
        else:
            return v[x]

    monkeypatch.setattr(os, 'getenv', patch_envar)

    c = read_aws_credentials()
    print(c)
    assert c['aws_access_key_id'] == v['LTD_MASON_AWS_ID']
    assert c['aws_secret_access_key'] == v['LTD_MASON_AWS_SECRET']
    assert 'aws_profile' not in c


def test_read_aws_credentials_profile(monkeypatch):
    """Test read_aws_credentials() if through a profile environment variable.
    """
    v = {'LTD_MASON_AWS_ID': 'key-id',
         'LTD_MASON_AWS_SECRET': 'key-secret',
         'LTD_MASON_AWS_PROFILE': 'aws-profile'}
    missing = ('LTD_MASON_AWS_ID', 'LTD_MASON_AWS_SECRET')

    def patch_envar(x):
        if x in missing:
            return None
        else:
            return v[x]

    monkeypatch.setattr(os, 'getenv', patch_envar)

    c = read_aws_credentials()
    print(c)
    assert c['aws_profile'] == v['LTD_MASON_AWS_PROFILE']
    assert 'aws_access_key_id' not in c
    assert 'aws_secret_access_key' not in c


@pytest.mark.parametrize('missing', [('LTD_MASON_AWS_ID',
                                      'LTD_MASON_AWS_SECRET',
                                      'LTD_MASON_AWS_PROFILE'),
                                     ('LTD_MASON_AWS_ID',
                                      'LTD_MASON_AWS_PROFILE'),
                                     ('LTD_MASON_AWS_SECRET',
                                      'LTD_MASON_AWS_PROFILE')])
def test_read_aws_credentials_fallback(monkeypatch, missing):
    """Test read_aws_credentials() if environment variables are missing,
    which causes a fallback to assume the user has default configs for boto3.
    """
    v = {'LTD_MASON_AWS_ID': 'key-id',
         'LTD_MASON_AWS_SECRET': 'key-secret',
         'LTD_MASON_AWS_PROFILE': 'aws-profile'}

    def patch_envar(x):
        if x in missing:
            return None
        else:
            return v[x]

    monkeypatch.setattr(os, 'getenv', patch_envar)

    c = read_aws_credentials()
    print(c)
    assert len(c) == 0


@pytest.fixture
def demo_manifest():
    resource_args = (__name__, 'demo_manifest.yaml')
    assert pkg_resources.resource_exists(*resource_args)
    yaml_data = pkg_resources.resource_string(*resource_args)
    manifest = Manifest(yaml_data)
    return manifest


@responses.activate
def test_register_build_ok(demo_manifest):
    """Test that _register_build returns the correct values."""
    expected_json = {
        "bucket_name": "an-s3-bucket",
        "bucket_root_dir": "lsst_apps/builds/b1",
        "date_created": "2016-03-01T10:21:27.583795Z",
        "date_ended": None,
        "git_refs": ["master"],
        "github_requester": "jonathansick",
        "product_url": "http://localhost:5000/products/lsst_apps",
        "self_url": "http://localhost:5000/builds/1",
        "slug": "b1",
        "surrogate_key": "35d7a50a1d1b40ab9e7a56cd169f356e",
        "uploaded": False}
    responses.add(
        responses.POST,
        'http://localhost:5000/products/lsst_apps/builds/',
        json=expected_json,
        status=201)

    build_info = _register_build(demo_manifest,
                                 'http://localhost:5000',
                                 'token')
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url \
        == 'http://localhost:5000/products/lsst_apps/builds/'
    for k, v in expected_json.items():
        assert build_info[k] == v


@responses.activate
def test_register_build_failed(demo_manifest):
    """Test that KeeperError is raised for a bad response."""
    expected_json = {
        "bucket_name": "an-s3-bucket",
        "bucket_root_dir": "lsst_apps/builds/b1",
        "date_created": "2016-03-01T10:21:27.583795Z",
        "date_ended": None,
        "git_refs": ["master"],
        "github_requester": "jonathansick",
        "product_url": "http://localhost:5000/products/lsst_apps",
        "self_url": "http://localhost:5000/builds/1",
        "slug": "b1",
        "uploaded": False}
    responses.add(
        responses.POST,
        'http://localhost:5000/products/lsst_apps/builds/',
        json=expected_json,
        status=404)

    with pytest.raises(KeeperError):
        _register_build(demo_manifest,
                        'http://localhost:5000',
                        'token')
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url \
        == 'http://localhost:5000/products/lsst_apps/builds/'


@responses.activate
def test_confirm_upload_ok():
    url = 'http://localhost:5000/builds/1'

    responses.add(
        responses.PATCH,
        url,
        json={'uploaded': True},
        status=200)

    _confirm_upload('http://localhost:5000/builds/1', 'token')
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url == url


@responses.activate
def test_confirm_upload_failed():
    """Check that an exception is raised when keeper does not return 200
    on an upload confirmation reques.
    """
    url = 'http://localhost:5000/builds/1'

    responses.add(
        responses.PATCH,
        url,
        json={'uploaded': True},
        status=404)

    with pytest.raises(KeeperError):
        _confirm_upload('http://localhost:5000/builds/1', 'token')
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url == url


def test_upload_via_keeper(demo_manifest, mocker):
    """Test upload_via_keeper.

    The goal is to ensure that it

    1. registers a build
    2. Calls upload with the right attributes
    3. Confirms the upload
    """
    build_resource = {
        "bucket_name": "an-s3-bucket",
        "bucket_root_dir": "lsst_apps/builds/b1",
        "date_created": "2016-03-01T10:21:27.583795Z",
        "date_ended": None,
        "git_refs": ["master"],
        "github_requester": "jonathansick",
        "product_url": "http://localhost:5000/products/lsst_apps",
        "self_url": "http://localhost:5000/builds/1",
        "slug": "b1",
        "surrogate_key": "35d7a50a1d1b40ab9e7a56cd169f356e",
        "uploaded": False}
    mock_register = mocker.patch('ltdmason.uploader._register_build')
    mock_register.return_value = build_resource
    mock_upload = mocker.patch('ltdmason.uploader.s3upload_upload')
    mock_confirm = mocker.patch('ltdmason.uploader._confirm_upload')

    mock_product = mock.MagicMock()
    mock_product.html_dir = '_build/html'

    upload_via_keeper(demo_manifest, mock_product,
                      'https://ltd-keeper.example.org', 'token')

    # Check that upload_via_keeper called the right things
    assert mock_register.call_count == 1
    assert mock_upload.call_count == 1
    assert mock_confirm.call_count == 1

    mock_register.assert_called_once_with(
        demo_manifest, 'https://ltd-keeper.example.org', 'token')

    mock_upload.assert_called_once_with(
        build_resource['bucket_name'],
        build_resource['bucket_root_dir'],
        mock_product.html_dir,
        surrogate_key=build_resource['surrogate_key'],
        acl=None,
        cache_control_max_age=31536000)

    mock_confirm.assert_called_once_with(build_resource['self_url'], 'token')
